@@ -34,6 +34,7 @@ class Question (object):
         'id',
         'prompt',
         'answer',
+        'accept_all',
         'multiline',
         'help',
         'dependencies',
@@ -57,21 +58,32 @@ class Question (object):
     def __setstate__(self, state):
         if 'id' not in state:
             state['id'] = state.get('prompt', None)
-        if 'multiline' not in state:
-            state['multiline'] = False
         if 'dependencies' not in state:
             state['dependencies'] = []
         if 'tags' not in state:
             state['tags'] = set()
         else:
             state['tags'] = set(state['tags'])
+        for attr in ['accept_all', 'multiline']:
+            if attr not in state:
+                state[attr] = False
         for attr in self._state_attributes:
             if attr not in state:
                 state[attr] = None
         self.__dict__.update(state)
 
     def check(self, answer):
-        return answer == self.answer
+        if self.accept_all:
+            return (True, None)
+        return self._check(answer)
+
+    def _check(self, answer):
+        correct = answer == self.answer
+        details = None
+        if not correct:
+            details = 'answer ({}) does not match expected value'.format(
+                answer)
+        return (correct, details)
 
     def _format_attribute(self, attribute, newline='\n'):
         value = getattr(self, attribute)
@@ -90,13 +102,38 @@ class NormalizedStringQuestion (Question):
     def normalize(self, string):
         return string.strip().lower()
 
-    def check(self, answer):
-        return self.normalize(answer) == self.normalize(self.answer)
+    def _check(self, answer):
+        normalized_answer = self.normalize(answer)
+        correct = normalized_answer == self.normalize(self.answer)
+        details = None
+        if not correct:
+            details = ('normalized answer ({}) does not match expected value'
+                       ).format(normalized_answer)
+        return (correct, details)
 
 
 class ChoiceQuestion (Question):
-    def check(self, answer):
-        return answer in self.answer
+    _state_attributes = Question._state_attributes + [
+        'display_choices',
+        'multiple_answers',
+        ]
+
+    def __setstate__(self, state):
+        for key in ['display_choices', 'multiple_answers']:
+            if key not in state:
+                state[key] = False
+        super(ChoiceQuestion, self).__setstate__(state)
+
+    def _check(self, answer):
+        if self.multiple_answers and not isinstance(answer, str):
+            correct = min([a in self.answer for a in answer])
+        else:
+            correct = answer in self.answer
+        details = None
+        if not correct:
+            details = 'answer ({}) is not in list of expected values'.format(
+                answer)
+        return (correct, details)
 
 
 class ScriptQuestion (Question):
@@ -202,12 +239,13 @@ class ScriptQuestion (Question):
         return (a_status,a_stdout,a_stderr,
                 t_status,t_stdout,t_stderr)
 
-    def check(self, answer=None, tempdir=None):
+    def _check(self, answer=None, tempdir=None):
         """Compare the user's answer with expected values
 
         Arguments are passed through to ._invoke() for calculating the
         user's response.
         """
+        details = None
         # figure out the expected values
         (ea_status,ea_stdout,ea_stderr,
          et_status,et_stdout,et_stderr) = self._invoke(answer=self.answer)
@@ -218,25 +256,29 @@ class ScriptQuestion (Question):
                 answer=answer, tempdir=tempdir)
         except (KeyboardInterrupt, _error.CommandError) as e:
             if isinstance(e, KeyboardInterrupt):
-                LOG.warning('KeyboardInterrupt')
+                details = 'KeyboardInterrupt'
             else:
-                LOG.warning(e)
-            return False
+                details = str(e)
+            return (False, details)
         # compare user-generated output with expected values
         if answer:
             if self.compare_answers:
-                if _util.invocation_difference(  # compare answers
-                        ea_status, ea_stdout, ea_stderr,
-                        ua_status, ua_stdout, ua_stderr):
-                    return False
+                difference = _util.invocation_difference(  # compare answers
+                    ea_status, ea_stdout, ea_stderr,
+                    ua_status, ua_stdout, ua_stderr)
+                if difference:
+                    details = _util.format_invocation_difference(*difference)
+                    return (False, details)
             elif ua_stderr:
                 LOG.warning(ua_stderr)
-        if _util.invocation_difference(  # compare teardown
-                et_status, et_stdout, et_stderr,
-                ut_status, ut_stdout, ut_stderr):
-            return False
-        return True
-        
+        difference = _util.invocation_difference(  # compare teardown
+            et_status, et_stdout, et_stderr,
+            ut_status, ut_stdout, ut_stderr)
+        if difference:
+            details = _util.format_invocation_difference(*difference)
+            return (False, details)
+        return (True, None)
+
 
 for name,obj in list(locals().items()):
     if name.startswith('_'):
