@@ -14,9 +14,15 @@
 # You should have received a copy of the GNU General Public License along with
 # quizzer.  If not, see <http://www.gnu.org/licenses/>.
 
+import collections as _collections
+import importlib as _importlib
+
 from .. import answerdb as _answerdb
 
 
+INTERFACES = ['cli', 'wsgi']
+
+
 class UserInterface (object):
     "Give a quiz over a generic user interface"
     def __init__(self, quiz=None, answers=None, stack=None):
@@ -27,23 +33,43 @@ class UserInterface (object):
         if stack is None:
             stack = self.answers.get_never_correctly_answered(
                 questions=quiz.leaf_questions())
-        self.stack = stack
+        self._stack = stack
+        self.stack = _collections.defaultdict(self._new_stack)
+
+    def _new_stack(self):
+        return list(self._stack)  # make a new copy for a new user
 
     def run(self):
         raise NotImplementedError()
 
-    def display_results(self):
-        raise NotImplementedError()
+    def get_question(self, user=None):
+        if self.stack[user]:
+            return self.stack[user].pop(0)
 
-    def get_question(self):
-        if self.stack:
-            return self.stack.pop(0)
-
-    def process_answer(self, question, answer, **kwargs):
-        correct = question.check(answer=answer, **kwargs)
-        self.answers.add(question=question, answer=answer, correct=correct)
+    def process_answer(self, question, answer, user=None, **kwargs):
+        correct,details = question.check(answer=answer, **kwargs)
+        self.answers.add(
+            question=question, answer=answer, correct=correct, user=user)
         if not correct:
-            self.stack.insert(0, question)
+            self.stack[user].insert(0, question)
             for qid in reversed(question.dependencies):
-                self.stack.insert(0, self.quiz.get(id=qid))
-        return correct
+                self.stack[user].insert(0, self.quiz.get(id=qid))
+        return (correct, details)
+
+
+def get_ui(name):
+    """Get the UserInterface subclass from a UI submodule
+
+    >>> get_ui('cli')
+    <class 'quizzer.ui.cli.CommandLineInterface'>
+    """
+    module = _importlib.import_module('{}.{}'.format(__name__, name))
+    for name in dir(module):
+        obj = getattr(module, name)
+        try:
+            subclass = issubclass(obj, UserInterface)
+        except TypeError as e:  # obj is not a class
+            continue
+        if subclass:
+            return obj
+    raise ValueError(name)
