@@ -28,7 +28,8 @@ except ImportError as e:
     print(e)
 
 from .. import error as _error
-from . import UserInterface
+from .. import question as _question
+from . import UserInterface as _UserInterface
 
 
 class QuestionCommandLine (_cmd.Cmd):
@@ -48,7 +49,7 @@ class QuestionCommandLine (_cmd.Cmd):
         self._tempdir = None
 
     def get_question(self):
-        self.question = self.ui.get_question()
+        self.question = self.ui.get_question(user=self.ui.user)
         if self.question:
             self._reset()
         else:
@@ -67,10 +68,16 @@ class QuestionCommandLine (_cmd.Cmd):
     def _set_ps1(self):
         "Pose a question and prompt"
         if self.question:
-            self.prompt = '\n{}\n{}'.format(
+            lines = [
+                '',
                 _colorize(
                     self.ui.colors['question'], self.question.format_prompt()),
-                _colorize(self.ui.colors['prompt'], self._prompt))
+                ]
+            lines.extend(
+                _colorize(self.ui.colors['prompt'], line)
+                for line in self._extra_ps1_lines())
+            lines.append(_colorize(self.ui.colors['prompt'], self._prompt))
+            self.prompt = '\n'.join(lines)
         else:
             self.prompt = _colorize(self.ui.colors['prompt'], self._prompt)
 
@@ -78,6 +85,44 @@ class QuestionCommandLine (_cmd.Cmd):
         "Just prompt (without the question, e.g. for multi-line answers)"
         self.prompt = _colorize(self.ui.colors['prompt'], self._prompt)
 
+    def _extra_ps1_lines(self):
+        if (isinstance(self.question, _question.ChoiceQuestion) and
+                self.question.display_choices):
+            for i,choice in enumerate(self.question.answer):
+                yield '{}) {}'.format(i, choice)
+            yield 'Answer with the index of your choice'
+            if self.question.accept_all:
+                conj = 'or'
+                if self.question.multiple_answers:
+                    conj = 'and/or'
+                yield '{} fill in an alternative answer'.format(conj)
+            if self.question.multiple_answers:
+                self._separator = ','
+                yield ("Separate multiple answers with the '{}' character"
+                       ).format(self._separator)
+        return []
+
+    def _process_answer(self, answer):
+        "Back out any mappings suggested by _extra_ps1_lines()"
+        if (isinstance(self.question, _question.ChoiceQuestion) and
+                self.question.display_choices):
+            if self.question.multiple_answers:
+                answers = []
+                for a in answer.split(self._separator):
+                    try:
+                        i = int(a)
+                        answers.append(self.question.answer[i])
+                    except (ValueError, IndexError):
+                        answers.append(a)
+                return answers
+            else:
+                try:
+                    i = int(answer)
+                    return self.question.answer[i]
+                except (ValueError, IndexError):
+                    pass
+        return answer
+
     def default(self, line):
         self.answers.append(line)
         if self.question.multiline:
@@ -95,15 +140,25 @@ class QuestionCommandLine (_cmd.Cmd):
             answer = self.answers[0]
         else:
             answer = ''
+        if answer == 'EOF':
+            return True  # quit
+        if answer == '':
+            return
         kwargs = {}
         if self._tempdir:
             kwargs['tempdir'] = self._tempdir
-        correct = self.ui.process_answer(
+        answer = self._process_answer(answer=answer)
+        correct,details = self.ui.process_answer(
             question=self.question, answer=answer, **kwargs)
         if correct:
             print(_colorize(self.ui.colors['correct'], 'correct\n'))
         else:
-            print(_colorize(self.ui.colors['incorrect'], 'incorrect\n'))
+            print(_colorize(self.ui.colors['incorrect'], 'incorrect'))
+            if details:
+                print(_colorize(
+                        self.ui.colors['incorrect'], '{}\n'.format(details)))
+            else:
+                print('')
         return self.get_question()
 
     def do_answer(self, arg):
@@ -149,7 +204,7 @@ class QuestionCommandLine (_cmd.Cmd):
 
     def do_skip(self, arg):
         "Skip the current question, and continue with the quiz"
-        self.ui.stack.append(self.question)
+        self.ui.stack[self.ui.user].append(self.question)
         return self.get_question()
 
     def do_hint(self, arg):
@@ -171,7 +226,7 @@ class QuestionCommandLine (_cmd.Cmd):
         super(QuestionCommandLine, self).do_help(arg)
 
 
-class CommandLineInterface (UserInterface):
+class CommandLineInterface (_UserInterface):
     colors = {  # listed in pygments.console.light_colors
         'question': 'turquoise',
         'prompt': 'blue',
@@ -181,22 +236,24 @@ class CommandLineInterface (UserInterface):
         }
 
     def run(self):
-        if not self.stack:
-            return
-        cmd = QuestionCommandLine(ui=self)
-        cmd.cmdloop()
-        print()
+        self.user = None
+        if self.stack[self.user]:
+            cmd = QuestionCommandLine(ui=self)
+            cmd.cmdloop()
+            print()
+        self._display_results()
 
-    def display_results(self):
+    def _display_results(self):
         print(_colorize(self.colors['result'], 'results:'))
+        answers = self.answers.get_answers(user=self.user)
         for question in self.quiz:
-            if question.id in self.answers:
-                self.display_result(question=question)
+            if question.id in answers:
+                self._display_result(question=question)
                 print()
-        self.display_totals()
+        self._display_totals()
 
-    def display_result(self, question):
-        answers = self.answers.get(question.id, [])
+    def _display_result(self, question):
+        answers = self.answers.get_answers(user=self.user).get(question.id, [])
         print('question:')
         print('  {}'.format(
             _colorize(
@@ -218,10 +275,11 @@ class CommandLineInterface (UserInterface):
             print('  you answered: {}'.format(ans))
             print('     which was: {}'.format(correct))
 
-    def display_totals(self):
-        answered = self.answers.get_answered(questions=self.quiz)
+    def _display_totals(self):
+        answered = self.answers.get_answered(
+            questions=self.quiz, user=self.user)
         correctly_answered = self.answers.get_correctly_answered(
-            questions=self.quiz)
+            questions=self.quiz, user=self.user)
         la = len(answered)
         lc = len(correctly_answered)
         print('answered {} of {} questions'.format(la, len(self.quiz)))
